%THETANOTE1
%   Computational experiment (tolerance) for the paper "On
%   well-separated sets and fast multipole methods". Graphics in
%   THETANOTEPLOTS.

% S. Engblom 2011-06-29 (Minor revision)
% S. Engblom 2011-01-28 (Revision)
% S. Engblom 2010-05-14

rand('state',20100514);

% number of potentials and radius
N = 1000000;
R = 1;

% uniform inside disc of radius R
z1 = R*sqrt(rand(N,1)).*exp(1i*2*pi*rand(N,1));

% non-uniform, density has 1/r singularity at the origin
z2 = R*rand(N,1).*exp(1i*2*pi*rand(N,1));

% mass
m = ones(N,1);
m = m/sum(m);

% M chosen checkpoints
M = 1000;
zx1 = z1(1:M);
px1 = fmm2d(z1,m,zx1,'ndirect',20,'pot',1,'tol',0);
zx2 = z2(1:M);
px2 = fmm2d(z2,m,zx2,'ndirect',20,'pot',1,'tol',0);

% sample error
err1 = zeros(30,3);
err2 = zeros(30,3);

warning('Platform-dependent make follows...');
itheta = 0;
for theta = [0.25 0.5 sqrt(0.5)]
  itheta = itheta+1;

  % this statement is platform-dependent, see make.m
  mx = mexext;
  if strcmp(mx,'mexa64') || strcmp(mx,'mexmaci64')
    make('-DBLASINT=size_t', ...
         '-DFMM_INLINECOMPLEX','-DFMM_INLINECOMPLEX_DIRECT', ...
         '-DFMM_SHRINK',sprintf('-DFMM_THETA=%.8f',theta));
  else
    make('-DFMM_INLINECOMPLEX','-DFMM_INLINECOMPLEX_DIRECT', ...
         '-DFMM_SHRINK',sprintf('-DFMM_THETA=%.8f',theta));
  end

  % observe errors
  if itheta == 1
    pp = 1:15;
  else
    pp = 1:30;
  end
  for p = pp
    tol = pow2(p*log2(theta))/4;
    floor(log2(4*tol)/log2(theta)-1)+1 % = p!

    % evaulate in checkpoints
    p1 = fmm2d(z1,m,zx1,'ndirect',20,'pot',1,'tol',tol);
    err1(p,itheta) = norm((p1-px1)./px1,inf);
    p2 = fmm2d(z2,m,zx2,'ndirect',20,'pot',1,'tol',tol);
    err2(p,itheta) = norm((p2-px2)./px2,inf);
  end
end

if exist('./examples/thetanote1.mat','file')
  fprintf('Existing file thetanote1.mat NOT overwritten.\n');
else
  save examples/thetanote1 err1 err2
end

make % compile back using defaults
