%THETANOTE3
%   Computational experiment (robustness) for the paper "On
%   well-separated sets and fast multipole methods". Graphics in
%   THETANOTEPLOTS.

% S. Engblom 2011-06-29 (Minor revision)
% S. Engblom 2011-02-05

rand('state',20110205);
nn = round(2.^(11:0.5:22));
t = zeros(3,size(nn,2));

% to get accurate timings and a smooth graph
Ntrymax = 17;
Ntrymin = 5;
Tmin = 5;

% build distributions
N = nn(end);
m = ones(N,1);
% (1) uniform
clear z
z{1} = complex(rand(N,1),rand(N,1));
% (2) normal N(0,1/10), forced and rejected to stay within the unit
% square
x = [0; 1];
y = [1; 0];
while numel(x) < N || numel(y) < N
  x = [x; abs(1/10*randn(N-numel(x),1))];
  y = [y; abs(1/10*randn(N-numel(y),1))];
  x(x > 1) = [];
  y(y > 1) = [];
end
z{2} = complex(x,y);
% (3) 'layered' distribution (combination of the above)
z{3} = complex(real(z{1}),imag(z{2}));

for i = 1:numel(nn)
  N = nn(i);
  for k = 1:3
    zz = z{k}(1:N); mm = m(1:N);
    j = 0;
    T = 0;
    tt = zeros(1,Ntrymax);
    while j < Ntrymin || (T < Tmin && j < Ntrymax)
      j = j+1;
      tt_ = cputime;
      p = fmm2d(zz,mm,'pot',1,'ndirect',20); % default tol = 1e-6
      tt(j) = cputime-tt_;
      T = T+tt(j);
    end
    % form a robust mean - "trimean with discarded outliers" - to get a
    % smooth graph
    tt = sort(tt(1:j)); tt([1 end]) = [];
    % [1:j-2] = [1:a a+1:b b+1:j-2] with the first and last interval of
    % equal length
    a = floor((j-2)/3); b = j-2-a;
    t(k,i) = 0.5*mean(tt([1:a b+1:end]))+0.5*mean(tt(a+1:b));
  end
  i
end

if exist('./examples/thetanote3.mat','file')
  fprintf('Existing file thetanote3.mat NOT overwritten.\n');
else
  save examples/thetanote3 t nn
end
