%THETANOTEPLOTS
%   Graphics for the paper "On well-separated sets and fast multipole
%   methods" using already computed values.

% S. Engblom 2011-01-28

load thetanote1 % already computed by THETANOTE1

% regenerate data (should match the corresponding lines in THETANOTE1)
rand('state',20100514);

% number of potentials and radius
N = 1000000;
R = 1;

% uniform inside disc of radius R
z1 = R*sqrt(rand(N,1)).*exp(1i*2*pi*rand(N,1));

% non-uniform, density has 1/r singularity at the origin
z2 = R*rand(N,1).*exp(1i*2*pi*rand(N,1));

% mass
m = ones(N,1);
m = m/sum(m);

% (1) max relative error as a function of order p, with theta = [0.25
% 0.5 sqrt(0.5)]
figure(1),
h = semilogy(1:15,err1(1:15,1),'r.-',1:15,err2(1:15,1),'bo-');
set(h(2),'MarkerSize',4);
hold on,
h = semilogy(1:30,err1(:,2),'r.-',1:30,err2(:,2),'bo-');
set(h(2),'MarkerSize',4);
h = semilogy(1:30,err1(:,3),'r.-',1:30,err2(:,3),'bo-');
set(h(2),'MarkerSize',4);
axis([0 30 1e-12 1])

% comparision slopes
plot([20 30],0.02*0.5.^[20 30],'k','LineWidth',2)
plot([20 30],0.2*sqrt(0.5).^[20 30],'k','LineWidth',2)
plot([12 20],0.1*0.25.^[12 20],'k','LineWidth',2)
h = text([18 25 25],[1e-11 2*1e-9 1e-4], ...
         {'\propto 0.25^p' '\propto 0.5^p' '\propto 0.5^{p/2}'});
set(h,'FontSize',10);

h = xlabel('Order p'); set(h,'FontSize',10);
h = ylabel('Max relative error'); set(h,'FontSize',10);
h = legend('Uniform','Non-uniform'); set(h,'FontSize',10);
set(gca,'xtick',0:10:30,'ytick',[1e-12 1e-8 1e-4 1]);
set(gcf,'PaperPositionMode','auto');
set(gcf,'Position',[100 100 400 247]);
%print -depsc errors.eps % uncomment to write to file

% (2) performance comparision assymmetric adaptive vs. uniform
load thetanote2
% speedup
su1 = tu1./ta1;
su2 = tu2./ta2;
figure(2),
h = semilogx(sigma,su1,'bo-',sigma,su2,'r.-');
set(h(1),'MarkerSize',4);
h = xlabel('\sigma'); set(h,'FontSize',10);
h = ylabel('Speedup'); set(h,'FontSize',10);
h = legend('Normal','Layer'); set(h,'FontSize',10);
axis([sigma(end) 1 0 8]);
set(gca,'Ytick',0:8);
hold on, plot(sigma([1 end]),[1 1],'k:');
set(gcf,'PaperPositionMode','auto');
set(gcf,'Position',[100 100 400 247]);
%print -depsc aavsu.eps % uncomment to write to file

% (3) performance robustness
load thetanote3
figure(3),
h = loglog(nn,t(1,:),'g-'); hold on
h = loglog(nn,t(2,:),'b--');
h = loglog(nn,t(3,:),'r-.');
axis([1e3 1e7 1e-2 1e2])
h = xlabel('Number of sources N'); set(h,'FontSize',10);
h = ylabel('CPU-Time (s)'); set(h,'FontSize',10);
h = legend('Uniform','Normal','Layer','Location','SouthEast');
set(h,'FontSize',10);
hold on, plot(nn([5 14]),0.25e-4*nn([5 14]),'k','LineWidth',2)
h = text(2e4,2,'\propto N');
set(h,'FontSize',10);
set(gcf,'PaperPositionMode','auto');
set(gcf,'Position',[100 100 400 247]);
%print -depsc robustness.eps % uncomment to write to file

% (4) illustrative plot of mesh
[M,C] = fmm2d(z2,m,'ndirect',20,'out','mesh');
l = 7;
figure(4), meshplot(M{l},'k'); hold on
C{l} = speye(size(C{l}))+C{l}''+C{l}';

% box i
i = 3500;
MM = M{l}(:,i);
z = MM([1 1 1 1],:)+[MM(2,:); conj(MM(2,:)); -MM(2,:); -conj(MM(2,:))];
fill(real(z),imag(z),'k');

% cluster-to-cluster interaction of box i
j = find(C{l}(:,i) == -1);
MM = M{l}(:,j);
z = MM([1 1 1 1],:)+[MM(2,:); conj(MM(2,:)); -MM(2,:); -conj(MM(2,:))];
fill(real(z),imag(z),[0.5 0.5 0.5]);

% here, optimized cross-level cluster-to-cluster interactions are
% displayed by plotting the receiving parent (a visual approximation)
j2 = find(C{l}(:,i) == -2);
j2 = floor((j2-1)/4)+1;
MM = M{l-1}(:,j2);
z = MM([1 1 1 1],:)+[MM(2,:); conj(MM(2,:)); -MM(2,:); -conj(MM(2,:))];
fill(real(z),imag(z),[0.5 0.5 0.5]);

% in this version all receiving boxes are plotted, this is in some
% sense more correct but does not display the true complexity
%j2 = find(C{l}(:,i) == -2);
%j2a = floor((j2(j2 > i)-1)/4)+1;
%MM = M{l-1}(:,j2a);
%z = MM([1 1 1 1],:)+[MM(2,:); conj(MM(2,:)); -MM(2,:); -conj(MM(2,:))];
%fill(real(z),imag(z),[0.5 0.5 0.5]);
%
%j2b = j2(j2 < i);
%MM = M{l}(:,j2b);
%z = MM([1 1 1 1],:)+[MM(2,:); conj(MM(2,:)); -MM(2,:); -conj(MM(2,:))];
%fill(real(z),imag(z),[0.5 0.5 0.5]);

axis square, axis([0 1 0 1]);
set(gca,'xtick',[],'ytick',[]);
set(gcf,'PaperPositionMode','auto');
set(gcf,'Position',[100 100 250 250]);
%print -deps mesh.eps % uncomment to write to file

% (5) illustration of Binary adaptivity vs. Asymmetric adaptivity
randn('state',20110201);
N = 4^3;
x = [0; 1];
y = [1; 0];
while numel(x) < N || numel(y) < N
  x = [abs(0.25*randn(N-numel(x),1)); x];
  y = [y; abs(0.25*randn(N-numel(y),1))];
  x(x > 1) = [];
  y(y > 1) = [];
end
z = complex(x,y);
m = ones(size(z));

% recompile with post-shrinking of boxes turned off, that is, no
% -DFMM_SHRINK (this statement is platform-dependent, see make.m)
warning('Platform-dependent make...');
mx = mexext;
if strcmp(mx,'mexa64') || strcmp(mx,'mexmaci64')
  make('-DBLASINT=size_t', ...
       '-DFMM_INLINECOMPLEX','-DFMM_INLINECOMPLEX_DIRECT');
else
  make('-DFMM_INLINECOMPLEX','-DFMM_INLINECOMPLEX_DIRECT');
end
[M1,C1] = fmm2d(z,m,'ndirect',1,'out','mesh');
make % compile back using defaults

% standard binary adaptivity (have to do this manually!)
M2 = [0.5+0.5i; 0.5+0.5i];
X = {z};
done = 0;
while ~done % until one potential in each box
  done = 1;
  MM = zeros(2,0);
  XX = cell(1,0);
  k = 1;
  for j = 1:size(M2,2)
    if numel(X{j}) > 1
      % vertical cut followed by a horizontal one
      MM(:,k:k+3) = [[M2(1,j)-0.5*M2(2,j) M2(1,j)+0.5*conj(M2(2,j)) ...
                      M2(1,j)-0.5*conj(M2(2,j)) M2(1,j)+0.5*M2(2,j)]; ...
                     [0.5*M2(2,j) 0.5*M2(2,j) 0.5*M2(2,j) 0.5*M2(2,j)]];
      XX(k:k+3) = {X{j}(real(X{j}) < real(M2(1,j)) ...
                        & imag(X{j}) < imag(M2(1,j))) ...
                   X{j}(real(X{j}) >= real(M2(1,j)) & ...
                        imag(X{j}) < imag(M2(1,j))) ...
                   X{j}(real(X{j}) < real(M2(1,j)) & ...
                        imag(X{j}) >= imag(M2(1,j))) ...
                   X{j}(real(X{j}) >= real(M2(1,j)) & ...
                        imag(X{j}) >= imag(M2(1,j)))};
      k = k+4;
      done = 0;
    else
      MM(:,k) = M2(:,j);
      XX(k) = X(j);
      k = k+1;
    end
  end
  M2 = MM;
  X = XX;
end

figure(5),
%plot(z,'k.');
hold on,
meshplot(M1{end},'r');
axis square
i = 57; % a suitable box
d = abs(M1{end}(1,:)-M1{end}(1,i));
R = sort([abs(M1{end}(2,:)); ...
          frepmat(abs(M1{end}(2,i)),[1 size(M1{end},2)])]);
r = R(1,:);
R = R(2,:);
j = find(R+0.5*r <= 0.5*d); % cluster-to-cluster
k = 1:size(M1{end},2); % all boxes
k(j) = []; % produces near-field boxes
meshplot(M1{end}(:,k),'k')

% interesting observation:
%CC = speye(size(C1{end}))+C1{end}+C1{end}';
%kk = find(CC(:,i) > 0)';
% kk differs from k by one box, the reason is explained in
% [@Lthetanoteplots] in fmm.c

MM = M1{end}(:,k);
zz = MM([1 1 1 1],:)+[MM(2,:); conj(MM(2,:)); -MM(2,:); -conj(MM(2,:))];
fill(real(zz),imag(zz),[0.5 0.5 0.5]);

MM = M1{end}(:,i);
zz = MM([1 1 1 1],:)+[MM(2,:); conj(MM(2,:)); -MM(2,:); -conj(MM(2,:))];
fill(real(zz),imag(zz),'k');
axis square, axis([0 1 0 1]);
set(gca,'xtick',[],'ytick',[]);
set(gcf,'PaperPositionMode','auto');
set(gcf,'Position',[100 100 250 250]);
%print -deps boxes2Daa.eps % uncomment to write to file

figure(6),
%plot(z,'k.');
hold on,
meshplot(M2,'b');
axis square
i = 106; % a suitable box
d = abs(M2(1,:)-M2(1,i));
R = sort([abs(M2(2,:)); ...
          frepmat(abs(M2(2,i)),[1 size(M2,2)])]);
r = R(1,:);
R = R(2,:);
j = find(R+0.5*r <= 0.5*d); % cluster-to-cluster
k = 1:size(M2,2); % all boxes
k(j) = []; % produces near-field boxes
meshplot(M2(:,k),'k')

MM = M2(:,k);
zz = MM([1 1 1 1],:)+[MM(2,:); conj(MM(2,:)); -MM(2,:); -conj(MM(2,:))];
fill(real(zz),imag(zz),[0.5 0.5 0.5]);

MM = M2(:,i);
zz = MM([1 1 1 1],:)+[MM(2,:); conj(MM(2,:)); -MM(2,:); -conj(MM(2,:))];
fill(real(zz),imag(zz),'k');

axis square, axis([0 1 0 1]);
set(gca,'xtick',[],'ytick',[]);
set(gcf,'PaperPositionMode','auto');
set(gcf,'Position',[100 100 250 250]);
%print -deps boxes2Dba.eps % uncomment to write to file

return;

% *** old version of paper ("1D" version)
%figure(3), clf
%x = [0.02 linspace(0.26,0.49,4) 0.98];
%dy = 1/25;
%y = linspace(-dy/2,dy/2,size(x,2));
%z = complex(x,y);
%m = ones(size(z));
%
%subplot(2,1,1);
%h = plot(x,0,'ko');
%set(h,'MarkerSize',5,'MarkerFaceColor','k','MarkerEdgeColor','k');
%hold on;
%
%% standard binary adaptivity
%M = [0.5; 0.5+1i*dy];
%X = {z};
%for i = 1:5
%  MM = zeros(2,0);
%  XX = cell(1,0);
%  k = 1;
%  for j = 1:size(M,2)
%    if numel(X{j}) > 1
%      MM(:,k:k+1) = [[real(M(1,j)-0.5*M(2,j))+1i*imag(M(1,j)); ...
%                      real(0.5*M(2,j))+1i*imag(M(2,j))] ...
%                     [real(M(1,j)+0.5*M(2,j))+1i*imag(M(1,j)); ...
%                      real(0.5*M(2,j))+1i*imag(M(2,j))]];
%      XX(k:k+1) = {X{j}(real(X{j}) < real(M(1,j))) ...
%                   X{j}(real(X{j}) >= real(M(1,j)))};
%      k = k+2;
%    else
%      MM(:,k) = M(:,j);
%      XX(k) = X(j);
%      k = k+1;
%    end
%  end
%  M = MM;
%  X = XX;
%end
%M1 = M;
%meshplot(M1,'k');
%axis equal, axis tight;
%set(gca,'xtick',[],'ytick',[]);
%
%ax2 = subplot(2,1,2);
%h = plot(x,0,'ko');
%set(h,'MarkerSize',5,'MarkerFaceColor','k','MarkerEdgeColor','k');
%hold on;
%
%% asymmetric adaptivity
%M = [0.5; 0.5+1i*dy];
%X = {z};
%for i = 1:3
%  MM = zeros(2,0);
%  XX = cell(1,0);
%  k = 1;
%  for j = 1:size(M,2)
%    med = median(real(X{j}));
%    % more "correct", but looks confusing:
%    %med = real(X{j}(1+floor(size(X{j},2)/2)));
%    MM(:,k:k+1) = [[0.5*(med+real(M(1,j)-M(2,j)))+1i*imag(M(1,j)); ...
%                    0.5*(med-real(M(1,j)-M(2,j)))+1i*imag(M(2,j))] ...
%                   [0.5*(real(M(1,j)+M(2,j))+med)+1i*imag(M(1,j)); ...
%                    0.5*(real(M(1,j)+M(2,j))-med)+1i*imag(M(2,j))]];
%    XX(k:k+1) = {X{j}(real(X{j}) < med) ...
%                 X{j}(real(X{j}) >= med)};
%    k = k+2;
%  end
%  M = MM;
%  X = XX;
%end
%M2 = M;
%meshplot(M2,'k');
%axis equal, axis tight;
%set(gca,'xtick',[],'ytick',[]);
%%p = get(ax2,'Position');
%%set(ax2,'Position',p);
%
%set(gcf,'PaperPositionMode','auto');
%set(gcf,'Position',[100 100 400 100]);
%%print -deps boxes.eps % uncomment to write to file
