function q = meshqual(M)
%MESHQUAL Compute mesh quality.
%   Q = MESHQUAL(M) computes two measures of the mesh quality of the
%   FMM mesh in M. Q(1) is the 1-norm quality and Q(2) the inf-norm.
%
%   A perfect mesh has a quality close to 1, a poor mesh close to 0.
%
%   See also MESHPLOT.

% S. Engblom 2009-05-10

% width to height
x = real(M(2,:))./imag(M(2,:));
x = min(x,1./x);
q = [norm(x,1)/size(M,2) norm(x,-inf)];
