function meshstats(M,C)
%MESHSTATS Display mesh statistics.
%   MESHSTATS(M,C) displays lots of interesting information of the
%   FMM-mesh M with connectivity matrix C.
%
%   No error-checking is performed.
%
%   See also FMM2D, MESHPLOT, MESHQUAL.

% S. Engblom 2010-03-10

nlevels = size(C,2);
fprintf('\n');
fprintf('FMM mesh statistics\n');
fprintf('-------------------\n\n');
fprintf(['Mesh: %d levels, %d boxes at the finest level, %d boxes ' ...
         'in total.\n\n'],nlevels,size(M{end},2),sum(cellfun('prodofsize',M)/2));

% expand to full format
if nnz(C{1}) == 0
  for l = 1:nlevels
    C{l} = speye(size(C{l}))+C{l}''+C{l}';
  end
end

fprintf('Connectivity per box (mean +- std):\n');
for l = 4:nlevels
  far = full(sum(C{l} == -1))+full(sum(C{l} == -2)/4);
  near = full(sum(C{l} == 1));
  if l == nlevels
    qfar = full(sum(C{l} == 2));
    fprintf(['  Level %2d: #far = (%5.2f +- %5.2f), ' ...
             '#near = (%5.2f +- %4.2f),\n'], ...
            l,mean(far),std(far),mean(near),std(near));
    fprintf('            #quasi-far = (%4.2f +- %4.2f).\n\n', ...
            mean(qfar),std(qfar));
  else
    fprintf(['  Level %2d: #far = (%5.2f +- %5.2f), ' ...
             '#near = (%5.2f +- %4.2f),\n'], ...
            l,mean(far),std(far),mean(near),std(near));
  end
end

fprintf('Miscellaneous information:\n');
for l = 4:nlevels
  r = abs(M{l}(2,:));
  [ii,jj] = find(C{l});
  theta0 = max(r(ii),r(jj)).^2+0.5*min(r(ii),r(jj)).^2 < ...
           0.5*0.5/(1+0.5)*abs(M{l}(1,ii)-M{l}(1,jj)).^2;
  theta1 = max(r(ii),r(jj))+0.5*min(r(ii),r(jj)) < ...
           0.5*abs(M{l}(1,ii)-M{l}(1,jj));
  fprintf(['  Level %2d: simplified theta-criterion positive %4.1f%%,\n' ...
           '            ditto false and should be false %4.1f%%,\n' ...
           '            ditto false and should be true %4.1f%%.\n'], ...
          l,100*mean(theta0), ...
          100*mean(~theta0 & ~theta1), ...
          100*mean(~theta0 & theta1));
end
fprintf('\n');
